#!/bin/bash

NAME=tester.sh
VERSION=0.0.1

while [[ "$#" -gt 0 ]]
do

    key="$1"

    case $key in
        --all)
        UART=YES
        I2C=YES
        POWER=YES
        SD_CARD=YES
        shift
        ;;
        --uart)
        UART=YES
        shift
        ;;
        --i2c)
        I2C=YES
        shift
        ;;
        --power)
        POWER=YES
        shift
        ;;
        --sd)
        SD_CARD=YES
        shift
        ;;
        -h|--help)
        HELP=YES
        shift
        ;;
        --version)
        SHOW_VERSION=YES
        shift
        ;;
        *)
        shift
        ;;
    esac

done

display_help() {
    echo "Usage: $NAME [OPTION]..."
    echo
    echo "OPTIONs:"
    echo
    echo "  --all         perform all checks"
    echo "  --uart        perform UART bus testing"
    echo "  --i2c         perform I2C testing"
    echo "  --power       perform power output testing"
    echo "  --sd          perform SD card testing"
    echo "  -h, --help    display this help message"
    echo "  --version     show version info"
    echo
    echo "OPTIONs may be combined, except for --help and --version"
    echo
    echo "Usage:"
    echo
    echo "$NAME --uart --i2c"
    echo "$NAME --i2c"
    echo "$NAME --version"
    echo
    echo "How to connect:"
    echo "UART:   In order to test UART bus connect UART0 and UART1"
    echo "        with special testing cable"
    echo "        (UART0_RX <-> UART1_TX, UART0_TX <-> UART1_RX)"
    echo "I2C:    In order to test I2C bus connect LiDAR to the I2C port"
    echo
}

if [ -v SHOW_VERSION ] ; then
    echo "$NAME $VERSION"
    exit 0
fi

if ! [ -v UART -o -v I2C -o -v POWER -o -v SD_CARD ] ; then
    HELP=YES
fi

if [ -v HELP ] ; then
    display_help
    exit 0
fi

#-------------------------------------
# UART
#-------------------------------------

function check_uart {
    local devout=$1
    local devin=$2
    local outdata=test
    local result="ERROR"
    local result_int=1
    local portoptions="-isig -icanon -iexten -echo -echoe -echok -echonl -noflsh -xcase -tostop
-echoprt -echoctl -echoke"
    {
        if [ -e $devout -a -e $devin ] ; then
            stty -F $devout 9600
            stty -F $devout raw
            stty -F $devout $portoptions
            stty -F $devin 9600
            stty -F $devin raw
            stty -F $devout $portoptions
            
            local buffer=$(mktemp)
            cat < "$devin" > "$buffer" &
            sleep 2
            pid="$!"
            echo "$outdata" > "$devout"
            sleep 2
            kill $pid
            local data=$(cat $buffer)
            if [ "$data" = "$outdata" ]; then
                result="OK"
                result_int=0
            fi
            rm $buffer
        fi
        echo $data
        echo $result
    } > /dev/null 2> /dev/null
    echo $result
    return $result_int
}

function test_uart() {
    echo "Checking UARTs..."

    /etc/init.d/skyhub stop || true
    sleep 3
   
    echo "From UART4 to UART6: $(check_uart /dev/ttymxc3 /dev/ttymxc5)"
    echo "From UART6 to UART4: $(check_uart /dev/ttymxc5 /dev/ttymxc3)"
}

if [ -v UART ] ; then
    test_uart
fi

#-------------------------------------
# I2C
#-------------------------------------

function check_i2c() {
    local address=$1
    local result
    {
        yes | i2cset 1 $address 0 2> /dev/null
        result=$(yes | i2cget 1 $address 0 w 2> /dev/null)
    } > /dev/null 2> /dev/null
    if [ $result ]; then
        local height=$(( ($result >> 8) + ($result & 0xFF) ))
        echo "OK, Value=$result, Height=$height"
        return 0
    else
        echo "ERROR"
        return 1
    fi
}

function test_i2c() {
    echo "Checking I2C bus..."

    /etc/init.d/skyhub stop || true
    sleep 2
    
    echo "I2C status is: $(check_i2c 0x66)"
}

if [ -v I2C ] ; then
    test_i2c
fi

#-------------------------------------
# Power output
#-------------------------------------

function test_power_output() {
    echo "Checking power output..."
    
    /etc/init.d/skyhub stop || true
    sleep 2
    
    echo "Initializing GPIO in high state"
    if [ ! -d /sys/class/gpio/gpio5 ]; then
        echo "Exporting GPIO"
        echo 5 > /sys/class/gpio/export
    fi
    echo high > /sys/class/gpio/gpio5/direction
    
    sleep 1
    echo "Transfer GPIO to LOW"
    echo 0 > /sys/class/gpio/gpio5/value
    echo "- Please check that red led is glowing"
    echo "- Check that power output is 12V"
    read -n 1 -s -r -p "Press any key to continue..."
    echo
    
    echo "Transfer GPIO to HIGH"
    echo 1 > /sys/class/gpio/gpio5/value
    echo "- Please check that red led is not glowing"
    echo "- Check that power output is 0V"
    read -n 1 -s -r -p "Press any key to continue..."
    echo
    
    echo "Power output check is over"
}

if [ -v POWER ] ; then
    test_power_output
fi

#-------------------------------------
# SD card
#-------------------------------------

function test_sd_card() {
    echo "Checking SD card..."
    
    /etc/init.d/skyhub stop || true
    sleep 2
    
    local status="OK"
    if [ ! -d /run/media/mmcblk1p1 ]; then
        status="ERROR"
        echo "- SD card is not present"
    fi
    
    echo "SD card:" $status
}

if [ -v SD_CARD ] ; then
    test_sd_card
fi
